unit Unit1;

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    ButOpen: TButton;
    ButClose: TButton;
    ButRun: TButton;
    ButStatus: TButton;
    ButImage: TButton;
    ButFlip: TButton;
    ButSet: TButton;
    ButVar: TButton;
    EditVar: TEdit;
    LabImage: TLabel;
    LabStatus: TLabel;
    LabVar: TLabel;
    MemoScript: TMemo;
    PaintBox1: TPaintBox;
    procedure ButCloseClick(Sender: TObject);
    procedure ButFlipClick(Sender: TObject);
    procedure ButImageClick(Sender: TObject);
    procedure ButOpenClick(Sender: TObject);
    procedure ButRunClick(Sender: TObject);
    procedure ButSetClick(Sender: TObject);
    procedure ButStatusClick(Sender: TObject);
    procedure ButVarClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
  private
    bmp: TBitmap;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
 AA_EXECUTE = 0;
 AA_TERMINATE = 1;
 AA_SCRIPT_RUN = 10;
 AA_SCRIPT_STOP = 11;
 AA_SCRIPT_GETSTATUS = 12;
 AA_SCRIPT_SHOW = 13;
 AA_SCRIPT_HIDE = 14;
 AA_SCRIPT_GETVAR = 20;
 AA_IMAGE_WIDTH = 30;
 AA_IMAGE_HEIGHT = 31;
 AA_IMAGE_UPDATE = 32;
 AA_IMAGE_PREPAREUNDO = 36;

 AA_SELECT_INTERFACE_62 = 5;
 AA_SELECT_INTERFACE_60 = 4;

 AARES_OK = $AA;
 AAS_IDLE = $A1;
 AAS_RUNNING = $A2;
 AAS_CAMWAIT = $A3;
 AAS_TELEWAIT = $A4;
 AAS_FOCUSERWAIT = $A5;
 AAS_GUIDESTARWAIT = $A6;

function AASendCommand(command:Integer; params:PAnsiChar): Integer;
         stdcall; external 'AARemote.dll';

function AAReadScriptVariable(pName:PAnsiChar): PAnsiChar;
         stdcall; external 'AARemote.dll';

function AAGetPixel(x,y:Integer): Integer;
         stdcall; external 'AARemote.dll';

function AASetPixel(x,y,v:Integer): Integer; stdcall;
         stdcall; external 'AARemote.dll';


procedure TForm1.FormCreate(Sender: TObject);
begin
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf32bit;
  bmp.Width := 16;
  bmp.Height := 16;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  bmp.Free;
end;

procedure TForm1.ButOpenClick(Sender: TObject);
begin
  AASendCommand(AA_EXECUTE,nil);
  AASendCommand(AA_SELECT_INTERFACE_62, nil); // disable this for 6.0
end;

procedure TForm1.ButCloseClick(Sender: TObject);
begin
  AASendCommand(AA_TERMINATE,nil);
end;

function CanRunScript: Boolean;
var status: Integer;
begin
  status := AASendCommand(AA_SCRIPT_GETSTATUS,nil);
  if status <> AAS_IDLE then begin
     ShowMessage('Error: the previous script is still running.');
     result := false;
  end
  else
     result := true;
end;

procedure TForm1.ButRunClick(Sender: TObject);
var s: AnsiString;
begin
  if not canRunScript then exit;
  s := MemoScript.Lines.Text;
  AASendCommand(AA_SCRIPT_RUN, PAnsiChar(s));
end;

function StatusToString(st:Integer): String;
begin
  case st of
    AAS_IDLE: result := 'Idle';
    AAS_RUNNING: result := 'Script running';
    AAS_CAMWAIT: result := 'Camera exposing';
    AAS_TELEWAIT: result := 'Telescope is moving';
    AAS_FOCUSERWAIT: result := 'Focuser is moving';
    AAS_GUIDESTARWAIT: result := 'Waiting for autoguide star';
    else result := 'Error!';
  end;
end;

procedure TForm1.ButStatusClick(Sender: TObject);
var
 status: Integer;
begin
  status := AASendCommand(AA_SCRIPT_GETSTATUS,nil);
  labStatus.Caption := StatusToString(status);
end;

procedure TForm1.ButVarClick(Sender: TObject);
var
 varName: AnsiString;
begin
  varName := EditVar.Text;
  LabVar.Caption := '= ' + AAReadScriptVariable(PAnsiChar(varName));
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
  PaintBox1.Canvas.FillRect(PaintBox1.ClientRect);
  PaintBox1.Canvas.Draw(0,0,bmp);
end;

function PBitmapPixels(bm: TBitmap): Pointer;
begin
  {$IFDEF FPC}
     result := Pointer(bm.RawImage.Data);
  {$ELSE}
     result := bm.ScanLine[bm.Height-1];
  {$ENDIF}
end;

procedure TForm1.ButImageClick(Sender: TObject);
var x,y,w,h,v: Integer;
begin
  w := AASendCommand(AA_IMAGE_WIDTH, nil);
  h := AASendCommand(AA_IMAGE_HEIGHT, nil);
  LabImage.Caption := Format('%d x %d',[w,h]);
  if w = 0 then
     exit;
  bmp.Width := w;
  bmp.Height := h;
  for y := 0 to h-1 do begin
      for x := 0 to w-1 do begin
           v := AAGetPixel(x, y);
           v := v div 50;              // this is a very simple visualization
           if v > 255 then v := 255;   // for 16 bit grayscale images
           bmp.Canvas.Pixels[x, h-1-y] := v * $010101;
           // Canvas.Pixels is VERY slow, use ScanLines for real projects
      end;
  end;
  PaintBox1.Invalidate;
end;

procedure TForm1.ButFlipClick(Sender: TObject);
begin
  if not CanRunScript then exit;
  AASendCommand(AA_SCRIPT_RUN, 'Image.FlipH');
  ButImage.Click;
end;

procedure TForm1.ButSetClick(Sender: TObject);
var
 w,h,x,y: Integer;
 v: Single;
begin
  w := AASendCommand(AA_IMAGE_WIDTH, nil);
  h := AASendCommand(AA_IMAGE_HEIGHT, nil);
  if w = 0 then
     exit;
  for y := 0 to h div 2 do
  for x := 0 to w div 2 do begin
      v := Sqrt(x*x + y*y) * 30.0;
      AASetPixel(x,y,Round(v));
  end;
  AASendCommand(AA_IMAGE_UPDATE,nil);
end;


end.

